pub struct EthStorageChip<'chip, F: Field> {
pub mpt: &'chip MPTChip<'chip, F>,
pub network: Option<Chain>,
}Expand description
Chip to prove correctness of account and storage proofs
Fields
mpt: &'chip MPTChip<'chip, F>network: Option<Chain>The network to use for block header decoding. Must be provided if using functions that prove into block header (as opposed to state / storage root)
Implementations
Sourceimpl<'chip, F: Field> EthStorageChip<'chip, F>
impl<'chip, F: Field> EthStorageChip<'chip, F>
pub fn new(mpt: &'chip MPTChip<'chip, F>, network: Option<Chain>) -> Self
pub fn gate(&self) -> &GateChip<F>
pub fn range(&self) -> &RangeChip<F>
pub fn rlc(&self) -> &RlcChip<F>
pub fn rlp(&self) -> RlpChip<'_, F>
pub fn keccak(&self) -> &KeccakChip<F>
Sourcepub fn parse_account_proof_phase0(
&self,
ctx: &mut Context<F>,
address: SafeAddress<F>,
proof: MPTProof<F>,
) -> EthAccountWitness<F>
pub fn parse_account_proof_phase0( &self, ctx: &mut Context<F>, address: SafeAddress<F>, proof: MPTProof<F>, ) -> EthAccountWitness<F>
Does inclusion/exclusion proof of key = keccak(addr) into an alleged MPT state trie. Alleged means the proof is with respect to an alleged stateRoot.
RLP decodes the ethereumAccount.
There is one global state trie, and it is updated every time a client processes a block. In it, a path is always: keccak256(ethereumAddress) and a value is always: rlp(ethereumAccount). More specifically an ethereum account is a 4 item array of [nonce,balance,storageRoot,codeHash].
Does input validation of proof (e.g., checking witnesses are bytes).
Sourcepub fn parse_account_proof_phase1(
&self,
(ctx_gate, ctx_rlc): RlcContextPair<'_, F>,
witness: EthAccountWitness<F>,
) -> EthAccountTrace<F>
pub fn parse_account_proof_phase1( &self, (ctx_gate, ctx_rlc): RlcContextPair<'_, F>, witness: EthAccountWitness<F>, ) -> EthAccountTrace<F>
SecondPhase of account proof parsing.
Sourcepub fn parse_account_proofs_phase0(
&self,
builder: &mut RlcCircuitBuilder<F>,
addr_proofs: Vec<(SafeAddress<F>, MPTProof<F>)>,
) -> Vec<EthAccountWitness<F>>
pub fn parse_account_proofs_phase0( &self, builder: &mut RlcCircuitBuilder<F>, addr_proofs: Vec<(SafeAddress<F>, MPTProof<F>)>, ) -> Vec<EthAccountWitness<F>>
Does multiple calls to [parse_account_proof_phase0] in parallel.
Sourcepub fn parse_account_proofs_phase1(
&self,
builder: &mut RlcCircuitBuilder<F>,
acct_witness: Vec<EthAccountWitness<F>>,
) -> Vec<EthAccountTrace<F>>
pub fn parse_account_proofs_phase1( &self, builder: &mut RlcCircuitBuilder<F>, acct_witness: Vec<EthAccountWitness<F>>, ) -> Vec<EthAccountTrace<F>>
SecondPhase of account proofs parsing.
Sourcepub fn parse_storage_proof_phase0(
&self,
ctx: &mut Context<F>,
slot: SafeBytes32<F>,
proof: MPTProof<F>,
) -> EthStorageWitness<F>
pub fn parse_storage_proof_phase0( &self, ctx: &mut Context<F>, slot: SafeBytes32<F>, proof: MPTProof<F>, ) -> EthStorageWitness<F>
Does inclusion/exclusion proof of keccak(slot) into an alleged MPT storage trie. Alleged means the proof is with respect to an alleged storageRoot.
storageRoot: A 256-bit hash of the root node of a Merkle Patricia tree that encodes the storage contents of the account (a mapping between 256-bit integer values), encoded into the trie as a mapping from the Keccak 256-bit hash of the 256-bit integer keys to the RLP-encoded 256-bit integer values. The hash is formally denoted σ[a]_s.
Will do input validation on proof (e.g., checking witnesses are bytes).
Sourcepub fn parse_storage_proof_phase1(
&self,
(ctx_gate, ctx_rlc): RlcContextPair<'_, F>,
witness: EthStorageWitness<F>,
) -> EthStorageTrace<F>
pub fn parse_storage_proof_phase1( &self, (ctx_gate, ctx_rlc): RlcContextPair<'_, F>, witness: EthStorageWitness<F>, ) -> EthStorageTrace<F>
SecondPhase of storage proof parsing.
Sourcepub fn parse_storage_proofs_phase0(
&self,
builder: &mut RlcCircuitBuilder<F>,
slot_proofs: Vec<(SafeBytes32<F>, MPTProof<F>)>,
) -> Vec<EthStorageWitness<F>>
pub fn parse_storage_proofs_phase0( &self, builder: &mut RlcCircuitBuilder<F>, slot_proofs: Vec<(SafeBytes32<F>, MPTProof<F>)>, ) -> Vec<EthStorageWitness<F>>
Does multiple calls to [parse_storage_proof_phase0] in parallel.
Sourcepub fn parse_storage_proofs_phase1(
&self,
builder: &mut RlcCircuitBuilder<F>,
storage_witness: Vec<EthStorageWitness<F>>,
) -> Vec<EthStorageTrace<F>>
pub fn parse_storage_proofs_phase1( &self, builder: &mut RlcCircuitBuilder<F>, storage_witness: Vec<EthStorageWitness<F>>, ) -> Vec<EthStorageTrace<F>>
SecondPhase of account proofs parsing.
Sourcepub fn parse_eip1186_proofs_phase0(
&self,
builder: &mut RlcCircuitBuilder<F>,
addr: SafeAddress<F>,
acct_pf: MPTProof<F>,
storage_pfs: Vec<(SafeBytes32<F>, MPTProof<F>)>,
) -> (EthAccountWitness<F>, Vec<EthStorageWitness<F>>)
pub fn parse_eip1186_proofs_phase0( &self, builder: &mut RlcCircuitBuilder<F>, addr: SafeAddress<F>, acct_pf: MPTProof<F>, storage_pfs: Vec<(SafeBytes32<F>, MPTProof<F>)>, ) -> (EthAccountWitness<F>, Vec<EthStorageWitness<F>>)
Does inclusion/exclusion proof of key = keccak(addr) into an alleged MPT state trie. Alleged means the proof is with respect to an alleged stateRoot.
RLP decodes the ethereumAccount, which in particular gives the storageRoot.
Does (multiple) inclusion/exclusion proof of keccak(slot) into the MPT storage trie with root storageRoot.
Sourcepub fn parse_eip1186_proofs_phase1(
&self,
builder: &mut RlcCircuitBuilder<F>,
(acct_witness, storage_witness): (EthAccountWitness<F>, Vec<EthStorageWitness<F>>),
) -> (EthAccountTrace<F>, Vec<EthStorageTrace<F>>)
pub fn parse_eip1186_proofs_phase1( &self, builder: &mut RlcCircuitBuilder<F>, (acct_witness, storage_witness): (EthAccountWitness<F>, Vec<EthStorageWitness<F>>), ) -> (EthAccountTrace<F>, Vec<EthStorageTrace<F>>)
SecondPhase of parse_eip1186_proofs_phase0
Sourcepub fn parse_eip1186_proofs_from_block_phase0(
&self,
builder: &mut RlcCircuitBuilder<F>,
input: EthBlockStorageInputAssigned<F>,
) -> (EthBlockAccountStorageWitness<F>, EIP1186ResponseDigest<F>)
pub fn parse_eip1186_proofs_from_block_phase0( &self, builder: &mut RlcCircuitBuilder<F>, input: EthBlockStorageInputAssigned<F>, ) -> (EthBlockAccountStorageWitness<F>, EIP1186ResponseDigest<F>)
Proves (multiple) storage proofs into storageRoot, prove account proof of storageRoot into stateRoot, and proves stateRoot is an RLP decoded block header. Computes the block hash by hashing the RLP encoded block header. In other words, proves block, account, storage with respect to an alleged block hash.
pub fn parse_eip1186_proofs_from_block_phase1( &self, builder: &mut RlcCircuitBuilder<F>, witness: EthBlockAccountStorageWitness<F>, ) -> EthBlockAccountStorageTrace<F>